--_________________________________________________________________________________________________
--                                                                                                |
--                                       |masterFIP core|                                         |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                         masterfip_tx                                           |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         masterfip_tx.vhd                                                                  |
--                                                                                                |
-- Description  The unit groups the main actions that regard FIELDRIVE data transmission.         |
--              Figure 1 shows the main units/processes; the units TX SERIALIZER and TX OSC come  |
--              unmodified from the nanoFIP project.                                              |
--              Figure 2 shows the frame structure; note that the fmc_masterfip_core ignores      |
--              completely the notion of PDU_TYPE, LGTH, MPS, etc fields inside the PAYLOAD       |
--              part of the frame. It is the processor (MT) that is providing all the bytes inside|
--              the PAYLOAD frame field. On the other hand the FSS, CRC and FES fields are        |
--              generated by the masterfip_tx.                                                    |
--              The serialization starts upon a rising edge on the tx_start_p_i and upon the end  |
--              of the FES transmission the masterfip_tx activates the tx_end_p_o signal.         |
--                                                                                                |
--              o TX SERIALIZER   this unit comes unmodified from the nanoFIP project.            |
--                                It receives bytes from the processor trough the                 |
--                                tx_frame structure, encodes them in Manchester 2, adds the FSS, |
--                                CRC, FES fields and puts one by one bits to the FIELDRIVE       |
--                                output FD_TXD following the synchronization signals from the    |
--                                wf_tx_osc unit. It also generates the output FD_TXENA.          |
--                                                                                                |
--              o TX OSC          this unit comes unmodified from the nanoFIP project.            |
--                                It generates the output FD_TXCK as well as an array of pulses,  |
--                                tx_sched_p_buff, used for the synchronization of the            |
--                                TX SERIALIZER's actions.                                        |
--                                                                                                |
--              o DATA RETRIEVAL  this unit copies the bytes provided by the processor upon       |
--                                the activation of the signal tx_start.                          |
--                                                                                                |
--                                                      Mock Turtle                               |
--                      ___________________________________________________________               |
--                     |                       _______________________________     |              |
--                     |                      |                               |    |              |
--                     |                      |        DATA RETRIEVAL         |    |              |
--                     |                      |_______________________________|    |              |
--                     |                            \/              \/             |              |
--                     |                         _________    _______________      |              |
--                     |                        |         |  |               |     |              |
--                     |                        | BYTES_C |  |  SELECT BYTE  |     |              |
--                     |                        |_________|  |_______________|     |              |
--                     |                                            \/             |              |
--                     |   _____________      __________________________________   |              |
--                     |  |             |    |                                  |  |              |
--                     |  |    TX OSC   |  > |          TX SERIALIZER           |  |              |
--                     |  |             |    |                                  |  |              |
--                     |  |_____________|    |__________________________________|  |              |
--                     |___________________________________________________________|              |
--                                                            \/                                  |
--                  ___________________________________________________________________           |
--                0_____________________________FIELDBUS______________________________O           |
--                                                                                                |
--                                    Figure 1: Module architecture                               |
--                                                                                                |
--           ___________ ______  ________________________________________  ___________ _______    |
--          |____FSS____|_CTRL_||_____________..DATA/PAYLOAD..___________||____CRC____|__FES__|   |
--                                                                                                |
--                                  Figure 2: WorldFIP Frame structure                            |
--                                                                                                |
-- Authors      Evangelia Gousiou     (Evangelia.Gousiou@cern.ch)                                 |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                                      SOLDERPAD LICENSE                                         |
--                                   Copyright CERN 2014-2018                                     |
--                              ------------------------------------                              |
-- Copyright and related rights are licensed under the Solderpad Hardware License, Version 2.0    |
-- (the "License"); you may not use this file except in compliance with the License.              |
-- You may obtain a copy of the License at http://solderpad.org/licenses/SHL-2.0.                 |
-- Unless required by applicable law or agreed to in writing, software, hardware and materials    |
-- distributed under this License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR       |
-- CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language   |
-- governing permissions and limitations under the License.                                       |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                      Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions
-- Specific library
library work;
use work.masterFIP_pkg.all;  -- definitions of types, constants, entities
use work.wf_package.all;


--=================================================================================================
--                           Entity declaration for masterfip_tx
--=================================================================================================

entity masterfip_tx is
  generic
    (g_span    : integer := 32;     -- address span in bus interfaces
     g_width   : integer := 32;     -- data width in bus interfaces
     g_simul   : boolean := FALSE); -- set to TRUE when instantiated in a test-bench


port(
  -- INPUTS
   clk_i           : in std_logic;                    -- only one clk domain

   rst_i           : in std_logic;                    -- core rst, synched with clk_i

   speed_i         : in std_logic_vector(1 downto 0); -- WorldFIP bit rate

   tx_frame_i      : in tx_frame_t;                   -- frame PAYLOAD bytes;
                                                      -- structure with 67 words of 32-bit each,
                                                      -- able to house a frame of max length;
                                                      -- upon tx_start_p_i, tx_bytes_num_i are
                                                      -- copied locally to be serialized

   tx_bytes_num_i  : in std_logic_vector(C_FRAME_BYTES_CNT_LGTH-1 downto 0);
                                                      -- number of bytes to be serialized

   tx_ctrl_byte_i  : in std_logic_vector(C_BYTE_WIDTH-1 downto 0);
                                                      -- frame CTRL byte

   tx_start_p_i    : in std_logic;                    -- indication for the start of the serialization


  -- OUTPUTS
   tx_byte_index_o : out std_logic_vector(C_FRAME_BYTES_CNT_LGTH-1 downto 0);
                                                      -- indx of byte currently being serialized (0-262)

   tx_end_p_o      : out std_logic;                   -- 1-clk-tick long pulse upon termination of
                                                      -- transmission (after FES)

   tx_d_o          : out std_logic;                   -- FIELDRIVE transmitter data

   tx_ena_o        : out std_logic;                   -- FIELDRIVE transmitter enable

   tx_clk_o        : out std_logic);                  -- FIELDRIVE transmitter line driver half bit clk

end entity masterfip_tx;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture struc of masterfip_tx is

  -- frame bytes
  signal prod_bytes_c                         : std_logic_vector(C_FRAME_BYTES_CNT_LGTH-1 downto 0);
  signal zero                                 : unsigned(C_FRAME_BYTES_CNT_LGTH-1 downto 0) := (others => '0');
  signal ctrl_byte, tx_byte                   : std_logic_vector(C_BYTE_WIDTH-1 downto 0);
  signal prod_frame                           : tx_frame_t;
  signal word32_num                           : integer range 0 to C_MAX_FRAME_WORDS-1;
  signal word32                               : std_logic_vector(C_WORD_WIDTH-1 downto 0);
  -- bytes counter
  signal bytes_num                            : std_logic_vector(C_FRAME_BYTES_CNT_LGTH-1 downto 0);
  signal prod_data_lgth_match                 : std_logic;
  signal last_data_byte_p, last_data_byte_p_d : std_logic;
  -- signals for wf_tx_osc
  signal s_tx_clk_p_buff                      : std_logic_vector(c_TX_SCHED_BUFF_LGTH-1 downto 0);
  signal s_tx_osc_rst_p                       : std_logic;
  -- signals for wf_tx_serializer
  signal byte_request_accept_p                : std_logic;
  signal byte_request_accept_p_d1             : std_logic;
  signal byte_request_accept_p_d2             : std_logic;
  signal tx_byte_request_p                    : std_logic;
  signal last_data_byte_p_tmp                 : std_logic;
  signal byte_request_accept_p_tmp            : std_logic;


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

---------------------------------------------------------------------------------------------------
--                                    Copy of the frame data                                     --
---------------------------------------------------------------------------------------------------
-- Local copy of the data bytes upon the tx_start_p_i.
-- All the frame bytes are copied to local registers; like this the frame data remain stable
-- until the next tx_start_p_i arrives.

  p_data_retrieval: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        prod_frame <= ((others => (others => '0')));
        ctrl_byte  <= (others => '0');
        bytes_num  <= (others => '0');
      else
        if tx_start_p_i = '1' then
          prod_frame <= tx_frame_i;     -- copying of the DATA bytes
          ctrl_byte  <= tx_ctrl_byte_i; -- copying of the CTRL byte
          bytes_num  <= tx_bytes_num_i; -- num of bytes to serialize from the prod_frame
        end if;
		end if;
    end if;
  end process;


---------------------------------------------------------------------------------------------------
--                                   Sequential byte selection                                   --
---------------------------------------------------------------------------------------------------
-- The following processes are responsible for the selection of the next byte to be serialised.
-- Following the frame structure of Figure 2, after the FSS (which is hard-coded in the
-- wf_tx_serializer's state machine), the first byte to serialize is the CTRL byte that comes from
-- the dedicated register ctrl_byte. Then, bytes are selected one by one from the prod_frame words
-- array, starting from byte(0) of the prod_frame(0). The wf_tx_serializer's signal
-- tx_byte_request_p indicates the need to retrieve a new byte for serialization.

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Selection of one 32-bit word of the prod_frame words array
  p_select_word: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        word32_num   <= 0;
      else
        if (unsigned(prod_bytes_c)) mod 4 = 0 and tx_byte_request_p = '1' and unsigned(prod_bytes_c) > 0 then
          word32_num <= word32_num + 1;
        end if;
      end if;
    end if;
  end process;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Instantiation of an incr_counter for the counting of the number of the bytes that are
  -- being serialized.
  cmp_tx_bytes_cnt: incr_counter
  generic map(g_counter_lgth => C_FRAME_BYTES_CNT_LGTH)
  port map(
    clk_i             => clk_i,
    counter_reinit_i  => tx_start_p_i,
    counter_incr_i    => tx_byte_request_p,
    -------------------------------------------------------
    counter_o         => prod_bytes_c);
    -------------------------------------------------------
  tx_byte_index_o     <= prod_bytes_c;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- Selection of one byte
  word32  <= prod_frame(word32_num);
  tx_byte <= ctrl_byte           when unsigned(prod_bytes_c) = zero    else -- first byte to serialize is the CTRL
            word32(7 downto 0)   when unsigned(prod_bytes_c) mod 4 = 1 else -- byte(0) (lsb) of a prod_frame word
            word32(15 downto 8)  when unsigned(prod_bytes_c) mod 4 = 2 else -- byte(1) of a prod_frame word
            word32(23 downto 16) when unsigned(prod_bytes_c) mod 4 = 3 else -- byte(2) of a prod_frame word
            word32(31 downto 24);                                           -- byte(3) (msb) of a prod_frame word


---------------------------------------------------------------------------------------------------
--                                 TX as in the nanoFIP project                                 --
---------------------------------------------------------------------------------------------------
  cmp_tx_serializer: wf_tx_serializer
  port map(
    uclk_i                  => clk_i,
    nfip_rst_i              => rst_i,
    tx_start_p_i            => tx_start_p_i,
    byte_i                  => tx_byte,
    byte_request_accept_p_i => byte_request_accept_p,
    last_byte_p_i           => last_data_byte_p,
    tx_sched_p_buff_i       => s_tx_clk_p_buff,
   -----------------------------------------------
    tx_byte_request_p_o     => tx_byte_request_p,
    tx_completed_p_o        => tx_end_p_o,
    tx_data_o               => tx_d_o,
    tx_osc_rst_p_o          => s_tx_osc_rst_p,
    tx_enable_o             => tx_ena_o);
   -----------------------------------------------

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  cmp_tx_osc: wf_tx_osc
  port map(
    uclk_i                  => clk_i,
    rate_i                  => speed_i,
    nfip_rst_i              => rst_i,
    tx_osc_rst_p_i          => s_tx_osc_rst_p,
   -----------------------------------------------
    tx_clk_o                => tx_clk_o,
    tx_sched_p_buff_o       => s_tx_clk_p_buff);
   -----------------------------------------------


---------------------------------------------------------------------------------------------------
--                              Signals essential to the serializer                              --
---------------------------------------------------------------------------------------------------

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- when s_prod_data_lgth bytes have been counted, the signal prod_data_lgth_match is activated
  prod_data_lgth_match <= '1' when prod_bytes_c = bytes_num else '0';

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  p_delay: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        last_data_byte_p         <= '0'; -- indication of the last DATA byte (CRC, FES not included)
        last_data_byte_p_d       <= '0';
        byte_request_accept_p_d1 <= '0';
        byte_request_accept_p_d2 <= '0';

      else
        last_data_byte_p_d       <= last_data_byte_p_tmp;
        last_data_byte_p         <= last_data_byte_p_d;

        byte_request_accept_p_d1 <= byte_request_accept_p_tmp;
        byte_request_accept_p_d2 <= byte_request_accept_p_d1;
      end if;
    end if;
  end process;
  --  --  --  --  --  --  --  --  --  --  --
  byte_request_accept_p_tmp <= tx_byte_request_p or tx_start_p_i;
  last_data_byte_p_tmp      <= prod_data_lgth_match and tx_byte_request_p;
  byte_request_accept_p     <= byte_request_accept_p_d2; -- response to wf_tx_serializer's request
                                                         -- for a byte

end architecture struc;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------